import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

public class Dictionary {
	private Map<String, Set<String>> words = new TreeMap<String, Set<String>>();

	/**
	 * Adds a word to the dictionary. If a correction is supplied, it is added
	 * to the word's correction list.
	 * 
	 * @pre word != null
	 * @pre word.length() > 0
	 * @post containsWord(word) == true
	 * @post correction != null $implies getCorrections(word) != null AND
	 *       correction is in getCorrections(word)
	 */
	public void addWordWithCorrection(String word, String correction) {
		Set<String> corrections = words.get(word);
		if (correction != null) {
			if (corrections == null) {
				corrections = new LinkedHashSet<String>();
				words.put(word, corrections);
			}
			corrections.add(correction);
		} else if (!words.containsKey(word)) {
			words.put(word, null);
		}
	}

	/**
	 * Adds a word to the dictionary.
	 * 
	 * @pre word != null
	 * @pre word.length() > 0
	 * @post containsWord(word) == true
	 */
	public void addWord(String word) {
		words.put(word, null);
	}

	/**
	 * Checks whether a word exists in the dictionary.
	 * 
	 * @pre word != null
	 * @post $ret == true iff word was added to the dictionary before
	 */
	public boolean containsWord(String word) {
		return words.containsKey(word);
	}

	/**
	 * Returns an array with the list of corrections added for this word
	 * 
	 * @pre word != null
	 * @post $ret != null
	 * @post for 0<=i< $ret.length, $ret[i] is a correction added to the dictionary for word
	 */
	public String[] getCorrections(String word) {
		Set<String> corrections = words.get(word);
		if (corrections == null) {
			return new String[0];
		} else {
			return corrections.toArray(new String[corrections.size()]);
		}
	}

}
